﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using UmlDiagramDesigner.ViewModels;

namespace UmlDiagramDesigner.DAL.Export
{
    public class ImageExporter : ExporterBase
    {
        protected override string FileFilter
        {
            get
            {
                return
                    "Png Image (.png)|*.png|Bitmap Image (.bmp)|*.bmp|Gif Image (.gif)|*.gif|JPEG Image (.jpeg)|*.jpeg|Tiff Image (.tiff)|*.tiff";
            }
        }

        public override void Export(CanvasViewModel diagram)
        {
            try
            {
                if (Canvas == null) return;

                var path = GetFilePath();

                if (string.IsNullOrEmpty(path)) return;

                var encoder = GetEncoderFromExtension(Path.GetExtension(SaveFileDialog.FileName));

                if (encoder == null) return;

                var bmpSource = new RenderTargetBitmap((int)Canvas.ActualWidth, (int)Canvas.ActualHeight, 96, 96, PixelFormats.Pbgra32);

                Rect bounds = VisualTreeHelper.GetDescendantBounds(Canvas);
                var dv = new DrawingVisual();

                using (DrawingContext ctx = dv.RenderOpen())
                {
                    var vb = new VisualBrush(Canvas);
                    ctx.DrawRectangle(vb, null, new Rect(new Point(), bounds.Size));
                }
                bmpSource.Render(dv);

                encoder.Frames.Add(BitmapFrame.Create(bmpSource));

                using (Stream stream = File.Create(path))
                {
                    encoder.Save(stream);
                    stream.Close();
                }
            }
            catch (Exception ex)
            {
                DialogService.Display(ex);
            }
        }

        private BitmapEncoder GetEncoderFromExtension(string extension)
        {
            switch (extension)
            {
                case ".bmp":
                    return new BmpBitmapEncoder();
                case ".jpeg":
                    return new JpegBitmapEncoder();
                case ".gif":
                    return new GifBitmapEncoder();
                case ".png":
                    return new PngBitmapEncoder();
                case ".tiff":
                    return new TiffBitmapEncoder();
                default:
                    return null;
            }
        }
    }
}
