﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Newtonsoft.Json;
using UmlDiagramDesigner.ViewModels;

namespace UmlDiagramDesigner.DAL.Serialization
{
    public class JsonSerializer : SerializerBase
    {
        protected override string FileFilter
        {
            get { return "Text file (.txt)|*.txt"; }
        }

        public override void Serialize(IEnumerable<DiagramEntityViewModelBase> diagram)
        {
            try
            {
                var path = GetNewFilePath();

                if (string.IsNullOrEmpty(path)) return;

                var serializer = new Newtonsoft.Json.JsonSerializer()
                {
                    TypeNameHandling = TypeNameHandling.Auto,
                    ReferenceLoopHandling = ReferenceLoopHandling.Serialize,
                    Formatting = Formatting.Indented,
                    PreserveReferencesHandling = PreserveReferencesHandling.Objects
                };

                using (var strWriter = new StreamWriter(path, false, Encoding.UTF8))
                {
                    serializer.Serialize(new JsonTextWriter(strWriter), diagram);
                }
            }
            catch (Exception ex)
            {
                DialogService.Display(ex);
            }
        }

        public override IEnumerable<DiagramEntityViewModelBase> Deserialize()
        {
            try
            {
                var path = GetOpenFilePath();

                if (string.IsNullOrEmpty(path)) return null;

                var serializer = new Newtonsoft.Json.JsonSerializer()
                {
                    TypeNameHandling = TypeNameHandling.Auto,
                    ReferenceLoopHandling = ReferenceLoopHandling.Serialize,
                    Formatting = Formatting.Indented,
                    PreserveReferencesHandling = PreserveReferencesHandling.Objects
                };

                using (var strReader =new StreamReader(path,Encoding.UTF8))
                {
                    return serializer.Deserialize<IEnumerable<DiagramEntityViewModelBase>>(new JsonTextReader(strReader));
                }
            }
            catch (Exception ex)
            {
                DialogService.Display(ex);
                return null;
            }
        }
    }
}
