﻿using System;
using System.Threading.Tasks;
using UmlDiagramDesigner.Core;
using UmlDiagramDesigner.Services;

namespace UmlDiagramDesigner.Helpers
{
    /// <summary>
    /// Singleton class for providing database access to document store of RavenDB.
    /// Also takes care of initializing and disposing of document store.
    /// </summary>
    public class DatabaseHelper
    {
        private readonly IDialogService _dialog = new WpfDialogService();

        private readonly static DatabaseHelper DatabaseHelperInstance = new DatabaseHelper();
        private DatabaseHelper() { }

        public static DatabaseHelper Instance { get { return DatabaseHelperInstance; } }

        public bool IsInitialized { get; private set; }

        //public EmbeddableDocumentStore DocumentStore { get; private set; }

        public Task InitializeDbAsync()
        {
            return Task.Factory.StartNew(() =>
            {
                try
                {
                    //DocumentStore = new EmbeddableDocumentStore
                    //{
                    //    DataDirectory = "Diagrams",
                    //    Conventions = { CustomizeJsonSerializer = serializer =>
                    //        {
                    //            serializer.TypeNameHandling = TypeNameHandling.Auto;
                    //            serializer.Formatting = Formatting.Indented;
                    //            serializer.ReferenceLoopHandling = ReferenceLoopHandling.Serialize;
                    //            serializer.PreserveReferencesHandling = PreserveReferencesHandling.Objects;
                    //        },
                    //    },
                    //};
                    //DocumentStore.Initialize();
                    IsInitialized = true;
                    RaiseInitializationCompletedEvent();
                }
                catch (Exception ex)
                {
                    _dialog.Display(ex);
                    DisposeDb();
                    RaiseInitializationFailedEvent();
                }
            });
        }

        public void DisposeDb()
        {
            //if (DocumentStore != null && IsInitialized)
            //{
            //    DocumentStore.Dispose();
            //    DocumentStore = null;
            //}
            IsInitialized = false;
        }

        private void RaiseInitializationCompletedEvent()
        {
            if(InitializationCompleted != null)
                InitializationCompleted(this,new EventArgs());
        }

        private void RaiseInitializationFailedEvent()
        {
            if(InitializationFailed != null)
                InitializationFailed(this,new EventArgs());
        }

        public event EventHandler InitializationCompleted;
        public event EventHandler InitializationFailed;
    }
}
