﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using Newtonsoft.Json;
using Newtonsoft.Json.Bson;
using UmlDiagramDesigner.ViewModels;

namespace UmlDiagramDesigner.Models
{
    public class ChangeTracker 
    {
        private static readonly ChangeTracker ChangeTrackerInstance = new ChangeTracker();
        private string _originalMd5;

        private ChangeTracker() { }

        public static ChangeTracker Instance { get { return ChangeTrackerInstance; } }

        public bool IsTracking { get { return _originalMd5 != null; } }

        public void Track(CanvasViewModel diagram)
        {
            _originalMd5 = GetMd5String(diagram);
        }

        public void StopTracking()
        {
            _originalMd5 = null;
        }

        private string GetMd5String(CanvasViewModel value)
        {
            if (value == null) return null;

            using (var md5 = MD5.Create())
            {
                var serializer = new JsonSerializer()
                {
                    TypeNameHandling = TypeNameHandling.Auto,
                    ReferenceLoopHandling = ReferenceLoopHandling.Serialize,
                    Formatting = Formatting.Indented,
                    PreserveReferencesHandling = PreserveReferencesHandling.Objects
                };
                using (var stream = new MemoryStream())
                {
                    serializer.Serialize(new BsonWriter(stream), value);
                    return Encoding.UTF8.GetString(md5.ComputeHash(stream.ToArray()));
                }
            }
        }

        public bool HasChanged(CanvasViewModel diagram)
        {
            if(_originalMd5 == null) throw new Exception("No value is tracker by ChangeTracker. Call Track() method first.");

            var currentMd5 = GetMd5String(diagram);

            return !currentMd5.Equals(_originalMd5);
        }
    }
}
