﻿using System.Collections.Generic;
using UmlDiagramDesigner.Core;
using UmlDiagramDesigner.Core.Graph;
using UmlDiagramDesigner.Models.PathFinders;

namespace UmlDiagramDesigner.Models.Connectors
{
    /// <summary>
    /// Defines an interface for implementation of an algorithm, which connects (links) associations together.
    /// Decision to connect two associations can be made based on different criteria, like a Type of the association.
    /// </summary>
    /// <typeparam name="TItem">The type of the item.</typeparam>
    public interface IAssociationConnector<TItem>
    {
        /// <summary>
        /// Gets or sets the path finder.
        /// </summary>
        /// <value>
        /// The path finder.
        /// </value>
        IPathFinder PathFinder { get; set; }

        /// <summary>
        /// Searches for suitable associations and connects them appropriately.
        /// </summary>
        void Connect(ICollection<IAssociation<TItem>> associations);
    }
}
