﻿using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using UmlDiagramDesigner.ViewModels.DependentElements;

namespace UmlDiagramDesigner.Models.DiagramValidation
{
    public class EntityValidator
    {
        public ValidationResult ValidateName(string entityName)
        {
            if (!Regex.IsMatch(entityName, @"^[a-zA-Z0-9_]+$"))
                return new ValidationResult(false)
                {
                    ErrorMessage =
                        "Name can contain only letters from english alphabet (no spaces, numbers or special characters)"
                };

            return new ValidationResult(true);
        }

        public ValidationResult Validate(ICollection<AttributeViewModel> attributes)
        {
            if (attributes.Any(att => !att.IsValid()))

                return new ValidationResult(false)
                {
                    ErrorMessage = "Attributes must have name and data type set."
                };

            if (attributes.Count(att => att.IsPrimaryKey) > 1)
            {
                return new ValidationResult(false)
                {
                    ErrorMessage = "Cannot set primary key constraint on multiple attributes."
                };
            }

            if (attributes.GroupBy(att => att.Name).Any(group => @group.Count() != 1))
            {
                return new ValidationResult(false)
                {
                    ErrorMessage = "There cannot be two or more attributes with the same name."
                };
            }
            return new ValidationResult(true);
        }

        public ValidationResult Validate(ICollection<TriggerViewModel> triggers)
        {
            if (triggers.GroupBy(trigger => trigger.Name).Any(group => @group.Count() != 1))
            {
                return new ValidationResult(false)
                {
                    ErrorMessage = "There cannot be two or more triggers with the same name"
                };
            }
            return new ValidationResult(true);
        }
    }
}
