﻿using System;
using System.Windows;
using UmlDiagramDesigner.Helpers;

namespace UmlDiagramDesigner.Models.PathFinders
{
    /// <summary>
    /// IPathFinder implementation, that searches for the shortest tuple of two
    /// points between two sets. It uses brute-force method so be careful about performance.
    /// </summary>
    public class ShortestPathPathFinder : PathFinderBase
    {
        private Random _rnd = new Random();

        public override Point[] FindPath(Point[] setA, Point[] setB)
        {
            var closestPoints = new Point[2];
            double tempClosest = double.MaxValue;

            foreach (var source in setA)
            {
                foreach (var target in setB)
                {
                    var tempDistance = source.CalculateDistance(target);
                    if (tempDistance < tempClosest)
                    {
                        tempClosest = tempDistance;
                        closestPoints[0] = source;
                        closestPoints[1] = target;
                    }
                }
            }

            return closestPoints;
        }

        public override Point[] FindPath(Point[] setA, Point[] setB, bool possibleStraightLine)
        {
            //optimization not yet implemented
            return FindPath(setA, setB);
        }
    }
}
