﻿using System;
using System.Windows;
using System.Windows.Media;
using UmlDiagramDesigner.Core.Enums;

namespace UmlDiagramDesigner.Models.Shapes
{
    public class AggregationShape : RelationshipShapeBase
    {
        private const int RhombWidth = 20;
        private const int RhombSharpAngle = 35;

        public AggregationShape()
        {
            EndShapeSize = 40;
            Fill = Brushes.Transparent;
        }

        protected override void DrawEndShape(StreamGeometryContext context, Point endShapePoint, RelativePosition? linePosition = null)
        {
            switch (linePosition)
            {
                case RelativePosition.OnTheLeft:
                    DrawLeftRhomb(context, endShapePoint);
                    break;
                case RelativePosition.OnTheRight:
                    DrawRightRhomb(context, endShapePoint);
                    break;
                case RelativePosition.Above:
                    DrawTopRhomb(context,endShapePoint);
                    break;
                case RelativePosition.Below:
                    DrawBottomRhomb(context,endShapePoint);
                    break;
                case RelativePosition.Away:
                case null:
                    break;
                default:
                    throw new ArgumentOutOfRangeException("linePosition");
            }
        }

        private void DrawLeftRhomb(StreamGeometryContext context, Point endShapePoint)
        {
            var startPoint = endShapePoint;
            var topPoint = new Point(startPoint.X + EndShapeSize/2, startPoint.Y - RhombWidth/2);
            var rightPoint = new Point(startPoint.X + EndShapeSize, startPoint.Y);
            var bottomPoint = new Point(startPoint.X + EndShapeSize/2, startPoint.Y + RhombWidth/2);

            context.BeginFigure(startPoint,true,false);
            context.LineTo(topPoint, true, true);
            context.LineTo(rightPoint, true, true);
            context.LineTo(bottomPoint, true, true);
            context.LineTo(startPoint, true, true);
        }

        private void DrawRightRhomb(StreamGeometryContext context, Point endShapePoint)
        {
            var startPoint = endShapePoint;
            var topPoint = new Point(startPoint.X - EndShapeSize/2, startPoint.Y - RhombWidth/2);
            var leftPoint = new Point(startPoint.X - EndShapeSize, startPoint.Y);
            var bottomPoint = new Point(startPoint.X - EndShapeSize/2, startPoint.Y + RhombWidth/2);

            context.BeginFigure(startPoint, true, false);
            context.LineTo(topPoint, true, true);
            context.LineTo(leftPoint, true, true);
            context.LineTo(bottomPoint, true, true);
            context.LineTo(startPoint, true, true);
        }
        private void DrawTopRhomb(StreamGeometryContext context, Point endShapePoint)
        {
            var startPoint = endShapePoint;
            var leftPoint = new Point(startPoint.X - RhombWidth/2, startPoint.Y - EndShapeSize/2);
            var topPoint = new Point(startPoint.X, startPoint.Y - EndShapeSize);
            var rightPoint = new Point(startPoint.X + RhombWidth/2, startPoint.Y - EndShapeSize/2);

            context.BeginFigure(startPoint, true, false);
            context.LineTo(leftPoint, true, true);
            context.LineTo(topPoint, true, true);
            context.LineTo(rightPoint, true, true);
            context.LineTo(startPoint, true, true);
        }

        private void DrawBottomRhomb(StreamGeometryContext context, Point endShapePoint)
        {
            var startPoint = endShapePoint;
            var leftPoint = new Point(startPoint.X - RhombWidth/2, startPoint.Y + EndShapeSize/2);
            var bottomPoint = new Point(startPoint.X, startPoint.Y + EndShapeSize);
            var rightPoint = new Point(startPoint.X + RhombWidth/2, startPoint.Y + EndShapeSize/2);

            context.BeginFigure(startPoint, true, false);
            context.LineTo(leftPoint, true, true);
            context.LineTo(bottomPoint, true, true);
            context.LineTo(rightPoint, true, true);
            context.LineTo(startPoint, true, true);
        }

        private double CalculateSideLength()
        {
            //p^2 + q^2 = 4a^2; p = height, q = width
            return Math.Sqrt(Math.Pow(EndShapeSize, 2) + Math.Pow(RhombWidth, 2))/4;
        }
    }
}
