﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using UmlDiagramDesigner.Core.Enums;

namespace UmlDiagramDesigner.Models.Shapes
{
    public class DependencyShape : RelationshipShapeBase
    {
        public DependencyShape()
        {
            EndShapeSize = 0;
            this.StrokeDashArray = new DoubleCollection(new Double[] {4,4});

            //no text can be drawn, because it would've been dashed as well
            SourceText = string.Empty;
            MiddleText = string.Empty;
            TargetText = string.Empty;
        }

        protected override void DrawEndShape(StreamGeometryContext context, Point endShapePoint, RelativePosition? linePosition = null)
        {
            switch (linePosition)
            {
                case RelativePosition.OnTheLeft:
                    DrawRightArrow(context,endShapePoint);
                    break;
                case RelativePosition.OnTheRight:
                    DrawLeftArrow(context, endShapePoint);
                    break;
                case RelativePosition.Above:
                    DrawTopArrow(context, endShapePoint);
                    break;
                case RelativePosition.Below:
                    DrawBottomArrow(context, endShapePoint);
                    break;
                case RelativePosition.Away:
                case null:
                    break;
                default:
                    throw new ArgumentOutOfRangeException("linePosition");
            }
        }


        private void DrawRightArrow(StreamGeometryContext context, Point endShapePoint)
        {
            var sideLength = CalculateTriangleSideLength();

            var startPoint = endShapePoint;
            var topPoint = new Point(startPoint.X - sideLength, startPoint.Y - sideLength/2);
            var bottomPoint = new Point(startPoint.X - sideLength, startPoint.Y + sideLength/2);

            context.BeginFigure(startPoint,true,true);
            context.LineTo(topPoint,true,true);
            context.LineTo(startPoint, true, true);
            context.LineTo(bottomPoint, true, true);
        }

        private void DrawLeftArrow(StreamGeometryContext context, Point endShapePoint)
        {
            var sideLength = CalculateTriangleSideLength();

            var startPoint = endShapePoint;
            var topPoint = new Point(startPoint.X + sideLength, startPoint.Y - sideLength / 2);
            var bottomPoint = new Point(startPoint.X + sideLength, startPoint.Y + sideLength / 2);

            context.BeginFigure(startPoint, true, true);
            context.LineTo(topPoint, true, true);
            context.LineTo(startPoint, true, true);
            context.LineTo(bottomPoint, true, true);
        }

        private void DrawTopArrow(StreamGeometryContext context, Point endShapePoint)
        {
            var sideLength = CalculateTriangleSideLength();

            var startPoint = endShapePoint;
            var leftPoint = new Point(startPoint.X - sideLength /2 , startPoint.Y + sideLength);
            var rightPoint = new Point(startPoint.X + sideLength/2, startPoint.Y + sideLength);

            context.BeginFigure(startPoint, true, true);
            context.LineTo(leftPoint, true, true);
            context.LineTo(startPoint, true, true);
            context.LineTo(rightPoint, true, true);
        }

        private void DrawBottomArrow(StreamGeometryContext context, Point endShapePoint)
        {
            var sideLength = CalculateTriangleSideLength();

            var startPoint = endShapePoint;
            var leftPoint = new Point(startPoint.X - sideLength / 2, startPoint.Y - sideLength);
            var rightPoint = new Point(startPoint.X + sideLength / 2, startPoint.Y - sideLength);

            context.BeginFigure(startPoint, true, true);
            context.LineTo(leftPoint, true, true);
            context.LineTo(startPoint, true, true);
            context.LineTo(rightPoint, true, true);
        }

        /// <summary>
        /// Calculates the length of the triangle side. 
        /// </summary>
        /// <returns>Side length.</returns>
        private double CalculateTriangleSideLength()
        {
            return (20 * Math.Sin((Math.PI / 180) * 90)) / Math.Sin((Math.PI / 180) * 60);
        }

        protected override void OnMouseLeave(object sender, MouseEventArgs mouseEventArgs)
        {
            this.Stroke = Brushes.Black;
            this.StrokeThickness = 2;
        }
    }
}
