﻿using System;
using System.Windows;
using System.Windows.Media;
using UmlDiagramDesigner.Core.Enums;

namespace UmlDiagramDesigner.Models.Shapes
{
    public class InheritanceShape : RelationshipShapeBase
    {
        public InheritanceShape()
        {
            EndShapeSize = 20;
            Fill = Brushes.Transparent;
            //v dedicnosti neni mozne aplikavat nejakou nasobnost (docasne reseni)
            SourceText = string.Empty;
            TargetText = string.Empty;
        }

        /// <summary>
        /// Calculates length of the triangle side from ArrowLength value.
        /// Triangle has all sides of same length.
        /// </summary>
        /// <returns></returns>
        private double CalculateTriangleSide()
        {
            return (EndShapeSize*Math.Sin((Math.PI/180)*90))/Math.Sin((Math.PI/180)*60);
        }

        #region End shape drawing

        protected override void DrawEndShape(StreamGeometryContext context, Point endShapePoint, RelativePosition? linePosition = null)
        {
            switch (linePosition)
            {
                case RelativePosition.OnTheLeft:
                    DrawRightTriangle(context, endShapePoint);
                    break;
                case RelativePosition.OnTheRight:
                    DrawLeftTriangle(context, endShapePoint);
                    break;
                case RelativePosition.Above:
                    DrawTopTriangle(context,endShapePoint);
                    break;
                case RelativePosition.Below:
                    DrawBottomTriangle(context,endShapePoint);
                    break;
                case null:
                case RelativePosition.Away:
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }

        private void DrawLeftTriangle(StreamGeometryContext context, Point endShapePoint)
        {
            var sideLength = CalculateTriangleSide();

            var startPoint = endShapePoint;
            var topCorner = new Point(startPoint.X, startPoint.Y - sideLength/2);
            var leftCorner = new Point(topCorner.X - sideLength, startPoint.Y);
            var bottomCorner = new Point(startPoint.X, leftCorner.Y + sideLength/2);

            context.BeginFigure(startPoint,true,false);
            context.LineTo(topCorner,true,true);
            context.LineTo(leftCorner, true, true);
            context.LineTo(bottomCorner,true,true);
            context.LineTo(startPoint,true,true);
        }

        private void DrawRightTriangle(StreamGeometryContext context, Point endShapePoint)
        {
            var sideLength = CalculateTriangleSide();

            var startPoint = endShapePoint;
            var topCorner = new Point(startPoint.X, startPoint.Y - sideLength / 2);
            var rightCorner = new Point(topCorner.X + sideLength, startPoint.Y);
            var bottomCorner = new Point(startPoint.X, rightCorner.Y + sideLength / 2);

            context.BeginFigure(startPoint, true, false);
            context.LineTo(topCorner, true, true);
            context.LineTo(rightCorner, true, true);
            context.LineTo(bottomCorner, true, true);
            context.LineTo(startPoint, true, true);
        }

        private void DrawTopTriangle(StreamGeometryContext context, Point endShapePoint)
        {
            var sideLength = CalculateTriangleSide();

            var startPoint = endShapePoint;
            var leftCorner = new Point(startPoint.X - sideLength/2, startPoint.Y);
            var topCorner = new Point(startPoint.X, startPoint.Y - sideLength);
            var rightCorner = new Point(startPoint.X + sideLength/2, startPoint.Y);

            context.BeginFigure(startPoint, true, false);
            context.LineTo(leftCorner, true, true);
            context.LineTo(topCorner, true, true);
            context.LineTo(rightCorner, true, true);
            context.LineTo(startPoint, true, true);
        }

        private void DrawBottomTriangle(StreamGeometryContext context, Point endShapePoint)
        {
            var sideLength = CalculateTriangleSide();

            var startPoint = endShapePoint;
            var leftCorner = new Point(startPoint.X - sideLength / 2, startPoint.Y);
            var bottomCorner = new Point(startPoint.X, startPoint.Y + sideLength);
            var rightCorner = new Point(startPoint.X + sideLength / 2, startPoint.Y);

            context.BeginFigure(startPoint, true, false);
            context.LineTo(leftCorner, true, true);
            context.LineTo(bottomCorner, true, true);
            context.LineTo(rightCorner, true, true);
            context.LineTo(startPoint, true, true);
        }

        #endregion

    }
}
