﻿using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using UmlDiagramDesigner.Helpers.Adorners;
using UmlDiagramDesigner.ViewModels;

namespace UmlDiagramDesigner.UserControls
{
    /// <summary>
    /// Interaction logic for DiagramCanvas.xaml
    /// </summary>
    public partial class DiagramCanvas : UserControl
    {
        private CanvasViewModel _canvasViewModel;
        private Point? _selectionStartPoint;
        private const double ScaleRate = 1.03;
        private bool _isUserSelecting;

        public DiagramCanvas()
        {
            InitializeComponent();
            _canvasViewModel = DiagramItemsControl.DataContext as CanvasViewModel;
            Instance = this;
        }

        public static DiagramCanvas Instance { get; private set; }

        public ItemsControl ItemsControlCanvas { get { return DiagramItemsControl; } }

    #region Event Handlers

        private void DiagramCanvas_OnDrop(object sender, DragEventArgs e)
        {
            if (_canvasViewModel == null)
                _canvasViewModel = DiagramItemsControl.DataContext as CanvasViewModel;
            if (_canvasViewModel == null) return;

            var item = e.Data.GetData(typeof(ToolboxItemViewModel)) as ToolboxItemViewModel;
            if (item == null) return;

            _canvasViewModel.CreateDroppedItemOnCanvas(item, e.GetPosition(DiagramItemsControl));
        }

        private void DiagramCanvasPreviewMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (_canvasViewModel == null)
                _canvasViewModel = DiagramItemsControl.DataContext as CanvasViewModel;
            if (_canvasViewModel == null) return;

            if (!_isUserSelecting) //user is not selecting with box selector
            {
                _canvasViewModel.DiagramCanvasMouseLeftButtonDownCommand.Execute(e.GetPosition(DiagramItemsControl));
            }

            var selectionAdorner = GetSelectionAdorner();
            if (selectionAdorner != null)
            {
                RemoveAdornerFromCanvas(selectionAdorner);
            }

            Focus(); //refocus to canvas so it can detect keybindings
        }

        private void DiagramItemsControl_OnPreviewMouseMove(object sender, MouseEventArgs e)
        {
            if (_canvasViewModel == null)
                _canvasViewModel = DiagramItemsControl.DataContext as CanvasViewModel;
            if (_canvasViewModel == null) return;

            _canvasViewModel.DiagramCanvasMouseMovedCommand.Execute(e.GetPosition(DiagramItemsControl));

            if (e.LeftButton != MouseButtonState.Pressed)
            {
                _selectionStartPoint = null;
            }

            if (_selectionStartPoint.HasValue && !_canvasViewModel.IsAnyItemHovered)
            {
                _isUserSelecting = true;
                ShowSelectionBox();
            }
            else
            {
                _isUserSelecting = false;
            }
        }

        private void DiagramItemsControl_OnPreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            base.OnPreviewMouseDown(e);

            //user is dragging an item from toolbox
            if (e.LeftButton == MouseButtonState.Pressed && e.Source.Equals(DiagramItemsControl))
            {
                _selectionStartPoint = e.GetPosition(DiagramItemsControl);
            }
        }

        #endregion

        #region Methods

        private void ShowSelectionBox()
        {
            var layer = AdornerLayer.GetAdornerLayer(DiagramItemsControl);
            if (layer == null) return;

            //adds selection box on the adorner layer of canvas
            var selectionAdorner = new SelectionBoxAdorner(DiagramItemsControl, _selectionStartPoint, _canvasViewModel);
            layer.Add(selectionAdorner);
        }

        private SelectionBoxAdorner GetSelectionAdorner()
        {
            var selectionBoxLayer = AdornerLayer.GetAdornerLayer(DiagramItemsControl);
            if (selectionBoxLayer != null)
            {
                var adorners = selectionBoxLayer.GetAdorners(DiagramItemsControl);
                if (adorners == null) return null;
                return adorners.OfType<SelectionBoxAdorner>().FirstOrDefault();
            }
            return null;
        }

        private void RemoveAdornerFromCanvas(Adorner adorner)
        {
            var adornerLayer = AdornerLayer.GetAdornerLayer(DiagramItemsControl);
            if (adornerLayer != null)
            {
                adornerLayer.Remove(adorner);
            }
        }

        private void HitTest(Point hitPoint)
        {
            var hitObject = InputHitTest(hitPoint) as DependencyObject;

            while (hitObject != null && hitObject.GetType() != typeof (ItemsControl))
            {
                hitObject = VisualTreeHelper.GetParent(hitObject);
            }
        }

        #endregion
    }
}
