﻿using System;
using UmlDiagramDesigner.Core.Enums;
using UmlDiagramDesigner.Models.Attributes;
using UmlDiagramDesigner.ViewModels.DependentElements;
using UmlDiagramDesigner.ViewModels.DiagramItems;
using UmlDiagramDesigner.ViewModels.InputForms;
using UmlDiagramDesigner.Views.InputForms;

namespace UmlDiagramDesigner.ViewModels.DiagramRelationships
{
    [AllowMultiple]
    //[RequiresConstraint(SqlConstraint.PrimaryKey | SqlConstraint.Unique)]
    public class AssociationViewModel : RelationshipViewModelBase, IForeignKeyRelationship
    {
        private RelationshipInputViewModel _inputViewModel;
        private bool _isBidirectional;
        private AttributeViewModel _sourceForeignKey;

        public AssociationViewModel()
        {
            CanBend = false;
            IsConnectable = false;
            IsBidirectional = false;
            SourceMultiplicity = MultiplicityType.One;
            TargetMultiplicity = MultiplicityType.ZeroToOne;
        }

        #region Properties

        public bool IsBidirectional
        {
            get { return _isBidirectional; }
            set { _isBidirectional = value; OnPropertyChanged();}
        }

        public AttributeViewModel SourceForeignKey
        {
            get { return _sourceForeignKey; }
            set
            {
                if (value.IsForeignKey)
                {
                    _sourceForeignKey = value;
                }
            }
        }

        #endregion

        public override string GetResourceStyleName()
        {
            return "AssociationStyle";
        }

        protected override void OpenEditWindow()
        {
            var window = new RelationshipEditWindow()
            {
                DataContext = _inputViewModel = new RelationshipInputViewModel(this)
            };

            _inputViewModel.InputCompleted += (sender, args) =>
            {
                SourceMultiplicity = _inputViewModel.SourceMultiplicity;
                MiddleText = _inputViewModel.MiddleText;
                TargetMultiplicity = _inputViewModel.TargetMultiplicity;
                AddifySourceForeignKey(_inputViewModel.SourceForeignKey);
            };

            window.ShowDialog();
        }

        /// <summary>
        /// Adds or modifies source foreign key
        /// </summary>
        private void AddifySourceForeignKey(AttributeViewModel foreignKey)
        {
            if (!foreignKey.IsForeignKey)
            {
                throw new ArgumentException("not a foreign key");
            }
            SourceForeignKey = foreignKey;

            var entity = Source as ClassDiagramItemViewModel;
            if (entity != null)
            {
                if (entity.Attributes.Contains(foreignKey))
                {
                    entity.Attributes.Remove(foreignKey);
                }
                entity.Attributes.Add(foreignKey);
            }
        }

        public override string ToString()
        {
            return "Association";
        }
    }
}
