﻿using System.Windows;
using System.Windows.Media;
using UmlDiagramDesigner.Properties;
using UmlDiagramDesigner.ViewModels.Commands;

namespace UmlDiagramDesigner.ViewModels
{
    public class SettingsViewModel : ViewModelBase
    {
        private Brush _selectedClassBackgroundColor;
        private Brush _selectedClassBorderColor;

        public SettingsViewModel()
        {
            SetCurrentValues();
        }

        #region Properties

        public Brush SelectedClassBackgroundColor
        {
            get { return _selectedClassBackgroundColor; }
            set { _selectedClassBackgroundColor = value; OnPropertyChanged();}
        }

        public Brush SelectedClassBorderColor
        {
            get { return _selectedClassBorderColor; }
            set { _selectedClassBorderColor = value; OnPropertyChanged();}
        }



        #endregion

        #region Commands

        public GenericCommand<SolidColorBrush> SetClassItemBorderBrushCommand
        {
            get { return new GenericCommand<SolidColorBrush>(brush =>
            {
                if (brush == null) return;

                Application.Current.Resources["ClassItemBorderBrush"] = brush;
            });}
        }

        public GenericCommand<SolidColorBrush> SetClassBackgroundBrushCommand
        {
            get { return new GenericCommand<SolidColorBrush>(brush =>
            {
                if (brush == null) return;

                Application.Current.Resources["ClassItemBackgroundBrush"] = brush;
            });}
        }

        public GenericCommand<Window> OkCommand
        {
            get { return new GenericCommand<Window>(window =>
            {
                Settings.Default.Save();
                window.Close();
            });}
        }

        public GenericCommand<Window> CancelCommand { get {return new GenericCommand<Window>(window =>
        {
            Settings.Default.Reload();
            window.Close();
        });} }

        #endregion

        #region Private methods

        private void SetCurrentValues()
        {
            var classBackground = Application.Current.Resources["ClassItemBackgroundBrush"] as SolidColorBrush;
            if (classBackground != null)
                SelectedClassBackgroundColor = classBackground;

            var classBorder = Application.Current.Resources["ClassItemBorderBrush"] as SolidColorBrush;
            if (classBorder != null)
                SelectedClassBorderColor = classBorder;

        }

        #endregion

    }
}
