﻿using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using UmlDiagramDesigner.ViewModels.Commands;
using UmlDiagramDesigner.ViewModels.DiagramItems;
using UmlDiagramDesigner.ViewModels.DiagramRelationships;

namespace UmlDiagramDesigner.ViewModels
{
    public class ToolboxViewModel : ViewModelBase
    {
        #region Variables

        private ObservableCollection<ToolboxItemViewModel> _diagramItems;
        private ObservableCollection<ToolboxItemViewModel> _diagramRelationships; 
        private ToolboxItemViewModel _selectedItem;

        #endregion

        #region Constructors

        public ToolboxViewModel()
        {
            Initialize();
            //determine if the UI is being editted in IDE
            if (DesignerProperties.GetIsInDesignMode(new DependencyObject()))
            {
                CreateTestToolboxItems();
            }
            CreateTestToolboxItems();
        }


        #endregion

        #region Properties

        public ObservableCollection<ToolboxItemViewModel> DiagramItems
        {
            get { return _diagramItems; }
            set
            {
                _diagramItems = value;
                OnPropertyChanged();
            }
        }

        public ObservableCollection<ToolboxItemViewModel> DiagramRelationships
        {
            get { return _diagramRelationships; }
            set
            {
                _diagramRelationships = value;
                OnPropertyChanged();
            }
        }

        public ToolboxItemViewModel SelectedItem
        {
            get { return _selectedItem; }
            set
            {
                _selectedItem = value;
                OnPropertyChanged();
            }
        }

        #endregion

        #region Private Methods

        private void Initialize()
        {
            _diagramItems = new ObservableCollection<ToolboxItemViewModel>();
            _diagramRelationships = new ObservableCollection<ToolboxItemViewModel>();
        }

        private void CreateTestToolboxItems()
        {
            _diagramItems.Add(
                new ToolboxItemViewModel("Class", "Represents a UML class.",
                    "../../Assets/Icons/class.jpg", typeof (ClassDiagramItemViewModel)));
            _diagramItems.Add(new ToolboxItemViewModel("View", "Represents view in relational databases.",
                "../../Assets/Icons/view.png", typeof(ViewDiagramItemViewModel)));
            _diagramItems.Add(new ToolboxItemViewModel("Comment", "Represents a relationship between two entities.",
                "../../Assets/Icons/comment.jpg", typeof(CommentDiagramItemViewModel)));

            _diagramRelationships.Add(new ToolboxItemViewModel("Inheritance", "Determines, that source is a subtype of target.",
                "../../Assets/Icons/generalization.jpg", typeof(InheritanceViewModel)));
            _diagramRelationships.Add(new ToolboxItemViewModel("Association", "Defines weak dependency between entities.",
                "../../Assets/Icons/association.png", typeof(AssociationViewModel)));
            _diagramRelationships.Add(new ToolboxItemViewModel("Composition", "Defines strong dependency of source entity to target entity.",
                "../../Assets/Icons/composition.jpg", typeof(CompositionViewModel)));
            _diagramRelationships.Add(new ToolboxItemViewModel("Aggregation", "Defines a \"Part of\" relationship between two entities.",
                "../../Assets/Icons/aggregation.jpg", typeof(AggregationViewModel)));
            _diagramRelationships.Add(new ToolboxItemViewModel("Dependency", "Determines, that source entity depends on target entity. Applicable on on view.",
                "../../Assets/Icons/dependency.jpg", typeof(DependencyViewModel)));
        }

        #endregion

        #region Commands

        public ICommand ItemSelectedCommand
        {
            get { return new GenericCommand<object>(item =>
            {
                var model = item as ToolboxItemViewModel;
                if (model != null)
                    SelectedItem = model;
            });}
        }

        public ICommand ItemUnselectedCommand
        {
            get { return new GenericCommand(() => SelectedItem = null);}
        }


        public ICommand StartDragCommand
        {
            get
            {
                return new GenericCommand<object>(parameter =>
                {
                    var itemsControl = parameter as ItemsControl;
                    if (SelectedItem != null && itemsControl != null)
                    {
                        DragDrop.DoDragDrop(itemsControl, SelectedItem, DragDropEffects.Copy);
                    }
                });
            }
        }

        #endregion
    }
}
