﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace UmlDiagramDesigner.CodeGeneration
{
    public static class ExtensionMethods
    {
		/// <summary>
		/// Inserts the tab characted at start of each line.
		/// </summary>
		/// <param name="input">The input.</param>
		/// <param name="tabCount">The tab count.</param>
		/// <returns></returns>
		public static string InsertTabsAtStartOfEachLine(this string input, int tabCount = 1)
        {
            using (StringReader reader = new StringReader(input))
            {
                string line;
                var sb = new StringBuilder();
                while ((line = reader.ReadLine()) != null)
                {
                    for (int i = 0; i < tabCount; i++)
                    {
                        line = line.Insert(0, "\t");
                    }
                    sb.AppendLine(line);
                }
                return sb.ToString();
            }
        }

		/// <summary>
		/// Converts <param name="input"></param> to pascal case.
		/// </summary>
		/// <param name="input">The input.</param>
		/// <returns></returns>
		public static string ToPascalCase(this string input)
        {
            TextInfo info = Thread.CurrentThread.CurrentCulture.TextInfo;
            input = info.ToTitleCase(input);
            string[] parts = input.Split(new char[] { },
                StringSplitOptions.RemoveEmptyEntries);
            string result = String.Join(String.Empty, parts);
            return result;
        }

		/// <summary>
		/// Converts <param name="input"></param> to camel case.
		/// </summary>
		/// <param name="input">The input.</param>
		/// <returns></returns>
		public static string ToCamelCase(this string input)
        {
            input = input.ToPascalCase();
            return input.Substring(0, 1).ToLower() +
                input.Substring(1);
        }

		/// <summary>
		/// Pluralizes the specified string.
		/// </summary>
		/// <param name="input">The input.</param>
		/// <returns></returns>
		public static string Pluralize(this string input)
        {
            return string.Format(input.EndsWith("s") ? "{0}es" : "{0}s", input);
        }
    }
}
