﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UmlDiagramDesigner.CodeGeneration.Contracts;
using UmlDiagramDesigner.CodeGeneration.Enums;
using UmlDiagramDesigner.Core.Enums;
using UmlDiagramDesigner.Data;

namespace UmlDiagramDesigner.CodeGeneration
{
	/// <summary>
	/// Concrete properties for java language.
	/// </summary>
	public class JavaProperties : IOopLanguageProperties
    {
        private Dictionary<DbDataType, string> _dataTypes;
        private Dictionary<LanguageFeature, string> _features;
        private Dictionary<Accessibility, string> _accessibilities;

        public Dictionary<LanguageFeature, string> Features
        {
            get { return _features ?? (_features = new Dictionary<LanguageFeature, string>()
            {
                {LanguageFeature.Comment, "//"},
                {LanguageFeature.BlockEnd, "}"},
                {LanguageFeature.BlockStart, "{"},
                {LanguageFeature.EndLineCharacter, ";"},
                {LanguageFeature.Inheritance, "extends"},
                {LanguageFeature.Namespace, "package"},
                {LanguageFeature.NamespaceImport, "import"},
            });}
        }

        public Dictionary<Accessibility, string> Accessibilities
        {
            get { return _accessibilities ?? (_accessibilities = new Dictionary<Accessibility, string>()
            {
                {Accessibility.Public, "public"},
                {Accessibility.Private, "private"},
                {Accessibility.Protected, "protected"}
            });}
        }

        public Dictionary<DbDataType, string> DataTypes
        {
            get { return _dataTypes ?? (_dataTypes = new Dictionary<DbDataType, string>()
            {
                {DbDataType.BigInteger, "BigInteger"},
                {DbDataType.Binary, "byte[]"},
                {DbDataType.Boolean, "boolean"},
                {DbDataType.Char, "String"},
                {DbDataType.Date, "Calendar"},
                {DbDataType.Decimal, "double"},
                {DbDataType.Float, "float"},
                {DbDataType.Integer, "int"},
                {DbDataType.SmallInteger, "short"},
                {DbDataType.Time, "Time"},
                {DbDataType.Timestamp, "Timestamp"},
                {DbDataType.Varchar, "String"},
            }); }
        }

        public string GenerateNavPropertyDataType(ForeignKeyConstraint navigationProperty)
        {
            switch (navigationProperty.TargetMultiplicity)
            {
                case MultiplicityType.One:
                    return navigationProperty.References.PartOf.Name;
                case MultiplicityType.ZeroToOne:
                    return navigationProperty.References.PartOf.Name;
                case MultiplicityType.ZeroToMany:
                    return string.Format("ArrayList<{0}>", navigationProperty.References.PartOf.Name);
                case MultiplicityType.OneToMany:
                    return string.Format("ArrayList<{0}>", navigationProperty.References.PartOf.Name);
                default:
                    throw new ArgumentOutOfRangeException("multiplicity");
            }
        }
    }
}
