﻿using System.Collections.Generic;
using System.Text;
using UmlDiagramDesigner.CodeGeneration.Contracts;
using UmlDiagramDesigner.CodeGeneration.Enums;
using UmlDiagramDesigner.Data;

namespace UmlDiagramDesigner.CodeGeneration
{
	/// <summary>
	/// Base table generator.
	/// </summary>
	public abstract class TableGeneratorBase : ISqlTableGenerator
    {
        public abstract string GenerateTables(IEnumerable<Class> classes);

        public ISqlProperties SqlProperties { get; set; }

		/// <summary>
		/// Generates the foreign key constraint.
		/// </summary>
		/// <param name="foreignKey">The foreign key.</param>
		/// <returns></returns>
		protected virtual string GenerateForeignKeyConstraint(ForeignKeyConstraint foreignKey)
        {
            var sb = new StringBuilder();

            sb.AppendLine(string.Format("ALTER TABLE {0}", ValidateString(foreignKey.PartOf.Name)));
            sb.AppendLine(string.Format("ADD FOREIGN KEY ({0})", ValidateString(foreignKey.Name)));
            sb.AppendLine(string.Format("REFERENCES {0}({1})", ValidateString(foreignKey.References.PartOf.Name),
                ValidateString(foreignKey.References.Name)));
            return sb.ToString();
        }

		/// <summary>
		/// Generates the trigger.
		/// </summary>
		/// <param name="trigger">The trigger.</param>
		/// <returns>generated trigger string</returns>
		protected virtual string GenerateTrigger(Trigger trigger)
        {
            var sb = new StringBuilder();

            sb.AppendLine(string.Format("CREATE TRIGGER {0} ON {1}", ValidateString(trigger.Name),
                ValidateString(trigger.OnTable.Name)));
            sb.AppendLine(SqlProperties.TriggerEvents[trigger.Event]);
            sb.AppendLine("AS");
            sb.AppendLine("BEGIN");
            sb.AppendLine(GenerateComment("Insert your trigger code here..."));
            sb.AppendLine("END");

            return sb.ToString();
        }

		/// <summary>
		/// Generates the comment.
		/// </summary>
		/// <param name="comment">The comment.</param>
		/// <returns>generated comment</returns>
		/// <exception cref="KeyNotFoundException">The property is retrieved and <paramref name="key" /> does not exist in the collection.</exception>
		protected virtual string GenerateComment(string comment)
        {
            return string.Format("{0} {1}", SqlProperties.Features[LanguageFeature.Comment], comment);
        }

        /// <summary>
        /// Validates the string for illegal words/characters.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns>Validated string</returns>
        protected virtual string ValidateString(string input)
        {
            if (SqlProperties.Keywords.Contains(input.ToUpperInvariant()))
            {
                return string.Format("{1}{0}{1}", input,
                    SqlProperties.Features[LanguageFeature.EscapeCharacter]);
            }
            return input;
        }

	    //protected virtual string GenerateConstraints([NotNull]IEnumerable<Constraint> constraints)
	    //{
		   // foreach (var constraint in constraints)
		   // {
			  //  SqlProperties.Constraints(constraint.GetType())
		   // }
	    //}
    }
}
