﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security;
using System.Text;
using System.Threading.Tasks;
using UmlDiagramDesigner.Core.Graph;

namespace UmlDiagramDesigner.Core
{
    public static class ExtensionMethods
    {
		/// <summary>
		/// Traverses grapth from <param name="start"/> with DFS algorithm.
		/// If revert is set to true, it goes in reverse direction.
		/// </summary>
		/// <param name="start">The start.</param>
		/// <param name="revert">if set to <c>true</c> [revert].</param>
		/// <returns></returns>
		public static IEnumerable<IVertex> DepthFirstTraversal(
            this IVertex start, bool revert = false)
        {
            var visited = new HashSet<IVertex>();
            var stack = new Stack<IVertex>();
			
            stack.Push(start);

            while (stack.Count != 0)
            {
                var current = stack.Pop();

                if (!visited.Add(current))
                    continue;

                yield return current;

                var neighbours = current.GetNeighbours(revert)
                    .Where(n => !visited.Contains(n));

                foreach (var neighbour in neighbours)
                    stack.Push(neighbour);
            }
        }
		/// <summary>
		/// Traverses grapth from <param name="start"/> with DFS algorithm.
		/// If revert is set to true, it goes in reverse direction.
		/// Generic parameter is used to filter edges that are used in traversal.
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="start">The start.</param>
		/// <param name="revert">if set to <c>true</c> [revert].</param>
		/// <returns></returns>
		/// <exception cref="System.ArgumentNullException">start</exception>
		public static IEnumerable<IVertex> DepthFirstTraversal<T>(
    this IVertex start, bool revert = false) where T : class
        {
			if(start == null) throw new ArgumentNullException("start");

            var visited = new HashSet<IVertex>();
            var stack = new Stack<IVertex>();

            stack.Push(start);

            while (stack.Count != 0)
            {
                var current = stack.Pop();

                if (!visited.Add(current)) continue;

                yield return current;

                var neighbours = current.GetNeighbours<T>(revert).Where(n => !visited.Contains(n));

                foreach (var neighbour in neighbours)
                    stack.Push(neighbour);
            }
        }
    }
}
