﻿using System.Collections.Generic;
using System.Windows;

namespace UmlDiagramDesigner.Core.Graph
{
	/// <summary>
	/// Defines an association (basically an edge) between two points (vertices).
	/// Generic argument is the item type which is connected by this association.
	/// </summary>
	/// <typeparam name="TItem">The type of the item.</typeparam>
	public interface IAssociation<TItem>
    {
		/// <summary>
		/// Gets or sets the source point.
		/// </summary>
		/// <value>
		/// The source point.
		/// </value>
		Point SourcePoint { get; set; }

		/// <summary>
		/// Gets or sets the target point.
		/// </summary>
		/// <value>
		/// The target point.
		/// </value>
		Point TargetPoint { get; set; }

		/// <summary>
		/// Gets or sets the source.
		/// </summary>
		/// <value>
		/// The source.
		/// </value>
		TItem Source { get; set; }
		/// <summary>
		/// Gets or sets the target.
		/// </summary>
		/// <value>
		/// The target.
		/// </value>
		TItem Target { get; set; }

		/// <summary>
		/// Gets or sets a value indicating whether this association is connected to another association, rather than directly to the target.
		/// </summary>
		/// <value>
		/// <c>true</c> if this instance is connected; otherwise, <c>false</c>.
		/// </value>
		bool IsConnected { get; set; }

		/// <summary>
		/// Gets a value indicating whether this instance is connectable to other associations.
		/// </summary>
		/// <value>
		/// <c>true</c> if this instance is connectable; otherwise, <c>false</c>.
		/// </value>
		bool IsConnectable { get; }

		/// <summary>
		/// Gets a value indicating whether this instance can bend. If set to false, diagonal connections has to be created by right angles.
		/// </summary>
		/// <value>
		///   <c>true</c> if this instance can bend; otherwise, <c>false</c>.
		/// </value>
		bool CanBend { get; }

		/// <summary>
		/// Gets or sets the additional points of the path.
		/// </summary>
		/// <value>
		/// The additional points.
		/// </value>
		IList<Point> AdditionalPoints { get; set; }
    }
}
