﻿using System.Collections.Generic;

namespace UmlDiagramDesigner.Core.Graph
{
	/// <summary>
	/// Defines indexed vertex. Addition properties like Index and LowLink are present.
	/// </summary>
	public interface IIndexedVertex : IVertex
    {
		/// <summary>
		/// Gets or sets the index.
		/// Used in <see cref="TarjansAlgorithm"/>
		/// </summary>
		/// <value>
		/// The index.
		/// </value>
		int Index { get; set; }
		/// <summary>
		/// Gets or sets the low link.
		/// Used in <see cref="TarjansAlgorithm"/>
		/// </summary>
		/// <value>
		/// The low link.
		/// </value>
		int LowLink { get; set; }
		/// <summary>
		/// Gets neighbour vertices.
		/// </summary>
		/// <param name="revert">if set to <c>true</c> [revert].</param>
		/// <returns></returns>
		IEnumerable<IIndexedVertex> GetIndexedNeighbours(bool revert = false);
		/// <summary>
		/// Gets the indexed neighbours connected with T edge.
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="revert">if set to <c>true</c> [revert].</param>
		/// <returns></returns>
		IEnumerable<IIndexedVertex> GetIndexedNeighbours<T>(bool revert = false) where T : class;
    }
}
