﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using UmlDiagramDesigner.Core.Enums;

namespace UmlDiagramDesigner.Core
{
    /// <summary>
    /// Handles point distribution.
    /// </summary>
    public class PointDistributor
    {
        private readonly List<Point> _topPoints = new List<Point>();
        private readonly List<Point> _bottomPoints = new List<Point>();
        private readonly List<Point> _leftPoints = new List<Point>();
        private readonly List<Point> _rightPoints = new List<Point>();

        private readonly IPointProvider _pointProvider;

        public PointDistributor(IPointProvider pointProvider)
        {
            _pointProvider = pointProvider;
        }

		/// <summary>
		/// Gets the point from provided position of the item.
		/// </summary>
		/// <param name="position">The position.</param>
		/// <returns></returns>
		/// <exception cref="System.ArgumentOutOfRangeException">position</exception>
		public Point GetPoint(RelativePosition position)
        {
            switch (position)
            {
                case RelativePosition.OnTheLeft:
                    return GetLeftPoint();
                case RelativePosition.OnTheRight:
                    return GetRightPoint();
                case RelativePosition.Above:
                    return GetTopPoint();
                case RelativePosition.Below:
                    return GetBottomPoint();
                default:
                    throw new ArgumentOutOfRangeException("position");
            }
        }

        private Point GetTopPoint()
        {
                var toReturn = _topPoints.First();
                _topPoints.Remove(toReturn);
                return toReturn;
        }

        private Point GetBottomPoint()
        {
                var toReturn = _bottomPoints.First();
                _bottomPoints.Remove(toReturn);
                return toReturn;
        }

        private Point GetLeftPoint()
        {
                var toReturn = _leftPoints.First();
                _leftPoints.Remove(toReturn);
                return toReturn;
        }

        private Point GetRightPoint()
        {
                var toReturn = _rightPoints.First();
                _rightPoints.Remove(toReturn);
                return toReturn;
        }

		/// <summary>
		/// Resets this instance.
		/// </summary>
		public void Reset()
        {
            _topPoints.Clear();
            _bottomPoints.Clear();
            _leftPoints.Clear();
            _rightPoints.Clear();

            _topPoints.AddRange(_pointProvider.GetAvailablePoints(RelativePosition.Above));
            _bottomPoints.AddRange(_pointProvider.GetAvailablePoints(RelativePosition.Below));
            _leftPoints.AddRange(_pointProvider.GetAvailablePoints(RelativePosition.OnTheLeft));
            _rightPoints.AddRange(_pointProvider.GetAvailablePoints(RelativePosition.OnTheRight));
        }
    }
}
